"""
class cybercops connection,

"""
import socket
import httplib
import urllib
from _errors import ConnectionError

class Connection(object):
    
    HTTP_PORT   =   80
    HTTPS_PORT  =  443
    NATIVE_PORT = 5000
    __HTTP_PATH = '/cybercops/?'
    __LN = "\n"
    __DEFAULT_TIMEOUT = 5.0
    
    def __init__(self, host=None, port=None, timeout=None):
        self.__host = host
        self.__port = port
        if timeout:
            self.__timeout = timeout
        else:
            self.__timeout = self.__DEFAULT_TIMEOUT
    
    def host():
        doc = "property of host"
        def fget(self): return self.__host
        def fset(self, val): self.__host = val
        return locals()
    host = property(**host())
    
    def port():
        doc = "property of port"
        def fget(self): return self.__port
        def fset(self, val): self.__port = int(val)
        return locals()
    port = property(**port())
    
    def timeout():
        def fget(self): return self.__timeout
        def fset(self, val): self.__timeout = float(val)
        return locals()
    timeout = property(**timeout())
    
    
    def request(self, param):
        try:
            org = socket.getdefaulttimeout()
            socket.setdefaulttimeout(self.__timeout)
            xml = None
            try:
                if self.__port in [self.HTTP_PORT, self.HTTPS_PORT]:
                    xml = self.__http_request((self.__port is self.HTTP_PORT \
                                               and httplib.HTTPConnection \
                                               or  httplib.HTTPSConnection), param)
                elif self.__port is self.NATIVE_PORT:
                    xml = self.__native_request(param)
                else:
                    raise ConnectionError('Port specification is illegal.')
            except socket.gaierror:
                raise ConnectionError("Name or service not known `%s`" % self.__host)
            except socket.error:
                raise ConnectionError("Connection refused")
            except socket.timeout:
                raise ConnectionError("Timeout %0.1fsec" % self.__timeout)
        finally:
            socket.setdefaulttimeout(org)
        return xml
    
    def __native_request(self, param):
        self.__s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.__s.connect((self.__host, self.__port))
            self.__sf = self.__s.makefile()
            nex = False
            nex = self.__send_data("BEGIN" + self.__LN)
            if nex:
                nex = self.__send_data(self.__make_auth_info(param))
            if nex:
                nex = self.__send_data("WORD:" + param['word'] + self.__LN)
            if nex:
                xml = self.__sf.read( self.__get_xmlsize() )
            else:
                xml = self.__sf.read(1024)
            self.__send_data("END" + self.__LN)
        finally:
            self.__s.close()
        return xml
        
    def __send_data(self, data):
        self.__s.send(str(len(data)) + self.__LN + data)
        firstline = self.__sf.readline().rstrip(self.__LN)
        if firstline is not "-1":
            return True
        else:
            return False
    
    def __get_xmlsize(self):
        line = self.__sf.readline().rstrip(self.__LN)
        return int(line)
    
    def __make_auth_info(self, param):
        buff = []
        for key, value in param.iteritems():
            if key is not "word":
                buff.append("%s:%s" % (key.upper(), str(value)))
        return self.__LN.join(buff) + self.__LN
    
    def __http_request(self, cls, param):
        conn = cls(self.__host, self.__port)
        try:
            conn.request("POST", self.__HTTP_PATH, urllib.urlencode(param))
            data = conn.getresponse().read()
        finally:
            conn.close()
        return data

if __name__ == '__main__':
    cls = Connection()
    for n in dir(cls): print n
